@@ -224,16 +224,10 @@ void HexBin::done(PointTableRef table)
         // Compute a UTM polygon
         BOX3D box = p.bounds();
         int zone = SpatialReference::calculateZone(box.minx, box.miny);
-
-        SpatialReference utm("EPSG:327" + std::to_string(std::abs(zone)));
-        density_p.transform(utm);
+        if (!density_p.transform(SpatialReference::wgs84FromZone(zone)))
+            density_p = Polygon();
     }
 
-    m_metadata.add("boundary", p.wkt(m_precision),
-        "Approximated MULTIPOLYGON of domain");
-    m_metadata.addWithType("boundary_json", p.json(), "json",
-        "Approximated MULTIPOLYGON of domain");
-
     double area = density_p.area();
     double density = m_count / area;
     if (std::isinf(density))
@@ -248,6 +242,11 @@ void HexBin::done(PointTableRef table)
     m_metadata.add("avg_pt_spacing", std::sqrt(1 / density),
         "Avg point spacing (x/y units)");
 
+    m_metadata.add("boundary", p.wkt(m_precision),
+        "Approximated MULTIPOLYGON of domain");
+    m_metadata.addWithType("boundary_json", p.json(), "json",
+        "Approximated MULTIPOLYGON of domain");
+
     int n(0);
     point_count_t totalCount(0);
     for (HexIter hi = m_grid->hexBegin(); hi != m_grid->hexEnd(); ++hi)
