@@ -35,6 +35,7 @@
 #include "TransformationFilter.hpp"
 #include <pdal/util/FileUtils.hpp>
 
+#include <Eigen/Dense>
 
 #include <sstream>
 
@@ -132,6 +133,7 @@ std::string TransformationFilter::getName() const { return s_info.name; }
 
 void TransformationFilter::addArgs(ProgramArgs& args)
 {
+    args.add("invert", "Apply inverse transformation", m_invert, false);
     args.add("matrix", "Transformation matrix", *m_matrix).setPositional();
     args.add("override_srs", "Spatial reference to apply to data.",
         m_overrideSrs);
@@ -142,6 +144,34 @@ void TransformationFilter::initialize()
 {
     if (! m_overrideSrs.empty())
         setSpatialReference(m_overrideSrs);
+
+    if (m_invert)
+    {
+        using namespace Eigen;
+
+        Transform& matrix = *m_matrix;
+
+        Affine3d T;
+        Matrix4d m;
+        m << matrix[0], matrix[1], matrix[2], matrix[3],
+             matrix[4], matrix[5], matrix[6], matrix[7],
+             matrix[8], matrix[9], matrix[10], matrix[11],
+             matrix[12], matrix[13], matrix[14], matrix[15];
+        T.matrix() = m;
+        Affine3d Tinv = T.inverse();
+        matrix[0] = Tinv.matrix()(0,0);
+        matrix[1] = Tinv.matrix()(0,1);
+        matrix[2] = Tinv.matrix()(0,2);
+        matrix[3] = Tinv.matrix()(0,3);
+        matrix[4] = Tinv.matrix()(1,0);
+        matrix[5] = Tinv.matrix()(1,1);
+        matrix[6] = Tinv.matrix()(1,2);
+        matrix[7] = Tinv.matrix()(1,3);
+        matrix[8] = Tinv.matrix()(2,0);
+        matrix[9] = Tinv.matrix()(2,1);
+        matrix[10] = Tinv.matrix()(2,2);
+        matrix[11] = Tinv.matrix()(2,3);
+    }
 }
 
 
