@@ -35,9 +35,8 @@
 
 #include <sstream>
 
-#include <pdal/EigenUtils.hpp>
-#include <pdal/GDALUtils.hpp>
 #include <pdal/PointView.hpp>
+#include <pdal/private/gdal/Raster.hpp>
 
 #include "private/GDALGrid.hpp"
 
@@ -152,8 +151,6 @@ void GDALWriter::initialize()
     // don't expand by point if we're running in standard mode.  That's
     // set later in writeView.
     m_expandByPoint = !m_fixedGrid;
-
-    gdal::registerDrivers();
 }
 
 
@@ -177,35 +174,26 @@ void GDALWriter::readyFile(const std::string& filename,
 }
 
 
-GDALWriter::Cell GDALWriter::cell(double x, double y)
+int GDALWriter::width() const
 {
-    Cell c;
-    c.x = static_cast<long>(std::floor((x - m_origin.x) / m_edgeLength));
-    c.y = static_cast<long>(std::floor((y - m_origin.y) / m_edgeLength));
-    return c;
+    return m_grid->width();
 }
 
 
-long GDALWriter::width() const
+int GDALWriter::height() const
 {
-    return static_cast<long>(m_grid->width());
-}
-
-
-long GDALWriter::height() const
-{
-    return static_cast<long>(m_grid->height());
+    return m_grid->height();
 }
 
 
 void GDALWriter::createGrid(BOX2D bounds)
 {
-    m_origin = { bounds.minx, bounds.miny };
-    Cell c = cell(bounds.maxx, bounds.maxy);
+    int width = std::floor((bounds.maxx - bounds.minx) / m_edgeLength) + 1;
+    int height = std::floor((bounds.maxy - bounds.miny) / m_edgeLength) + 1;
 
     try
     {
-        m_grid.reset(new GDALGrid(c.x + 1, c.y + 1, m_edgeLength,
+        m_grid.reset(new GDALGrid(bounds.minx, bounds.miny, width, height, m_edgeLength,
             m_radius, m_outputTypes, m_windowSize, m_power));
     }
     catch (GDALGrid::error& err)
@@ -215,37 +203,6 @@ void GDALWriter::createGrid(BOX2D bounds)
 }
 
 
-void GDALWriter::expandGrid(BOX2D bounds)
-{
-    Cell low = cell(bounds.minx, bounds.miny);
-    Cell high = cell(bounds.maxx, bounds.maxy);
-
-    long w = (std::max)(width(), high.x + 1);
-    long h = (std::max)(height(), high.y + 1);
-    long xshift = (std::max)(-low.x, 0L);
-    long yshift = (std::max)(-low.y, 0L);
-    if (xshift)
-    {
-        w += xshift;
-        m_origin.x -= xshift * m_edgeLength;
-    }
-    if (yshift)
-    {
-        h += yshift;
-        m_origin.y -= yshift * m_edgeLength;
-    }
-
-    try
-    {
-        m_grid->expand(w, h, xshift, yshift);
-    }
-    catch (const GDALGrid::error& err)
-    {
-        throwError(err.what()); // Add the stage name onto the error text.
-    }
-}
-
-
 void GDALWriter::writeView(const PointViewPtr view)
 {
     m_expandByPoint = false;
@@ -256,11 +213,14 @@ void GDALWriter::writeView(const PointViewPtr view)
     if (!m_fixedGrid)
     {
         BOX2D bounds;
-        calculateBounds(*view, bounds);
+        view->calculateBounds(bounds);
         if (!m_grid)
             createGrid(bounds);
         else
-            expandGrid(bounds);
+        {
+            m_grid->expandToInclude(bounds.minx, bounds.miny);
+            m_grid->expandToInclude(bounds.maxx, bounds.maxy);
+        }
     }
 
     PointRef point(*view, 0);
@@ -280,15 +240,11 @@ bool GDALWriter::processOne(PointRef& point)
 
     if (m_expandByPoint)
     {
-        Cell c = cell(x, y);
         if (!m_grid)
             createGrid(BOX2D(x, y, x, y));
-        else if (c.x < 0 || c.y < 0 ||
-                 c.x >= width() || c.y >= height())
-            expandGrid(BOX2D(x, y, x, y));
+        else
+            m_grid->expandToInclude(x, y);
     }
-    x -= m_origin.x;
-    y -= m_origin.y;
 
     m_grid->addPoint(x, y, z);
     return true;
@@ -303,10 +259,10 @@ void GDALWriter::doneFile()
 
     std::array<double, 6> pixelToPos;
 
-    pixelToPos[0] = m_origin.x;
+    pixelToPos[0] = m_grid->xOrigin();
     pixelToPos[1] = m_edgeLength;
     pixelToPos[2] = 0;
-    pixelToPos[3] = m_origin.y + (m_edgeLength * m_grid->height());
+    pixelToPos[3] = m_grid->yOrigin() + (m_edgeLength * m_grid->height());
     pixelToPos[4] = 0;
     pixelToPos[5] = -m_edgeLength;
     gdal::Raster raster(m_outputFilename, m_drivername, m_srs, pixelToPos);
