@@ -38,8 +38,9 @@
 namespace pdal
 {
 
-void Reader::readerAddArgs(ProgramArgs& args)
+void Reader::l_addArgs(ProgramArgs& args)
 {
+    Stage::l_addArgs(args);
     m_filenameArg = &args.add("filename", "Name of file to read", m_filename);
     m_countArg = &args.add("count", "Maximum number of points read", m_count,
         (std::numeric_limits<point_count_t>::max)());
@@ -76,8 +77,10 @@ void Reader::setSpatialReference(MetadataNode& m, const SpatialReference& srs)
 }
 
 
-void Reader::readerInitialize(PointTableRef)
+void Reader::l_initialize(PointTableRef table)
 {
+    Stage::l_initialize(table);
+
     if (m_overrideSrs.valid() && m_defaultSrs.valid())
         throwError("Cannot specify both 'override_srs' and 'default_srs'");
 
@@ -87,5 +90,10 @@ void Reader::readerInitialize(PointTableRef)
         setSpatialReference(m_defaultSrs);
 }
 
+void Reader::l_prepared(PointTableRef table)
+{
+    Stage::l_prepared(table);
+}
+
 } // namespace pdal
 
