@@ -380,7 +380,7 @@ TEST(SpatialReferenceTest, readerOptions)
 
         PointTable t;
         r.prepare(t);
-        EXPECT_EQ(r.getSpatialReference(), SpatialReference("EPSG:26916"));
+        EXPECT_EQ(r.getSpatialReference(), "EPSG:26916");
     }
 }
 
@@ -431,19 +431,16 @@ TEST(SpatialReferenceTest, merge)
 
 TEST(SpatialReferenceTest, test_bounds)
 {
-
     const std::string utm17_wkt = "PROJCS[\"WGS 84 / UTM zone 17N\",GEOGCS[\"WGS 84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199433,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]],PROJECTION[\"Transverse_Mercator\"],PARAMETER[\"latitude_of_origin\",0],PARAMETER[\"central_meridian\",-81],PARAMETER[\"scale_factor\",0.9996],PARAMETER[\"false_easting\",500000],PARAMETER[\"false_northing\",0],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],AXIS[\"Easting\",EAST],AXIS[\"Northing\",NORTH],AUTHORITY[\"EPSG\",\"32617\"]]";
 
     SpatialReference utm17(utm17_wkt);
 
     std::string wgs84_wkt = "GEOGCS[\"WGS 84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],AUTHORITY[\"EPSG\",\"6326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199433,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]]";
 
-
-    SpatialReference wgs84(wgs84_wkt);
     BOX2D box17(289814.15, 4320978.61, 289818.50, 4320980.59);
     pdal::Polygon p(box17);
     p.setSpatialReference(utm17);
-    p.transform(wgs84);
+    p.transform(wgs84_wkt);
 
     BOX3D b2 = p.bounds();
     EXPECT_FLOAT_EQ(static_cast<float>(b2.minx), -83.427597f);
