@@ -89,7 +89,7 @@ TEST(LasWriterTest, srs)
 
     LasTester tester;
     SpatialReference srs = tester.srs(writer);
-    EXPECT_EQ(srs, SpatialReference("EPSG:26915"));
+    EXPECT_EQ(srs, "EPSG:26915");
 }
 
 
@@ -114,7 +114,7 @@ TEST(LasWriterTest, srs2)
 
     LasTester tester;
     SpatialReference srs = tester.srs(writer);
-    EXPECT_EQ(srs, SpatialReference("EPSG:32615"));
+    EXPECT_EQ(srs, "EPSG:32615");
 }
 
 
@@ -1151,10 +1151,25 @@ TEST(LasWriterTest, pdal_add_vlr)
     Options readerOpts;
     readerOpts.add("filename", infile);
 
-    std::string vlr( " [ { \"description\": \"A description under 32 bytes\", \"record_id\": 42, \"user_id\": \"hobu\", \"data\": \"dGhpcyBpcyBzb21lIHRleHQ=\" },  { \"description\": \"A description under 32 bytes\", \"record_id\": 43, \"user_id\": \"hobu\", \"data\": \"dGhpcyBpcyBzb21lIG1vcmUgdGV4dA==\" } ]");
+    std::string vlr1(
+      R"({
+          "description": "A description under 32 bytes",
+          "record_id": 42,
+          "user_id": "hobu",
+          "data": "dGhpcyBpcyBzb21lIHRleHQ="
+         })"
+    );
+    std::string vlr2(
+      R"({
+          "description": "A description under 32 bytes",
+          "record_id": 43,
+          "user_id": "hobu",
+          "filename": ")" + Support::datapath("las/vlr-43.bin") + R"("})"
+    );
 
     Options writerOpts;
-    writerOpts.add("vlrs", vlr);
+    writerOpts.add("vlrs", vlr1);
+    writerOpts.add("vlrs", vlr2);
     writerOpts.add("filename", outfile);
 
     LasReader reader;
@@ -1179,10 +1194,148 @@ TEST(LasWriterTest, pdal_add_vlr)
 
     auto pred = [](MetadataNode temp)
         { return Utils::startsWith(temp.name(), "vlr_"); };
-    MetadataNodeList nodes = forward.findChildren(pred);
-    EXPECT_EQ(nodes.size(), 2UL);
+
+    MetadataNode root = reader2.getMetadata();
+    MetadataNodeList nodes = root.findChildren(pred);
+    EXPECT_EQ(nodes.size(), 2u);
+
+    MetadataNode node = nodes[1].findChild("data");
+    std::vector<uint8_t> buf =
+        Utils::base64_decode(node.value());
+
+
+    EXPECT_EQ(memcmp(buf.data(), "this is some more text",
+        buf.size() - 1), 0);
 }
 
+// Make sure we can read an array of VLRs in a pipeline.
+TEST(LasWriterTest, issue2937)
+{
+    std::string infile = Support::configuredpath("pipeline/issue2937.json");
+
+    std::string cmd = Support::binpath("pdal") + " pipeline --nostream " +
+        infile;
+    std::string output;
+    Utils::run_shell_command(cmd, output);
+
+    std::string outfile = Support::temppath("issue2937_out.las");
+    cmd = Support::binpath("pdal") + " info --metadata " + outfile;
+    Utils::run_shell_command(cmd, output);
+
+    EXPECT_NE(output.find("dGhpcyBpcy"), std::string::npos);
+    EXPECT_NE(output.find("dGhpcyAqdtB"), std::string::npos);
+}
+
+TEST(LasWriterTest, badVlr)
+{
+    auto doTest = [](const std::string& vlr, bool expectThrow = true)
+    {
+        Options opts;
+        opts.add("filename", Support::temppath("testfile"));
+        opts.add("vlrs", vlr);
+
+        LasWriter w;
+        w.setOptions(opts);
+
+        PointTable t;
+        if (expectThrow)
+            EXPECT_THROW(w.prepare(t), pdal_error);
+        else
+            EXPECT_NO_THROW(w.prepare(t));
+    };
+
+    doTest(
+      R"({
+        "description": "A description under 32 bytes",
+        "record_id": 42,
+        "user_id": "hobu",
+        "data": "dGhpcyBpcyBzb21lIHRleHQ="
+      })", false
+    );
+    doTest(
+      R"({
+        "description": "A description over 32 bytes is one that's way too long",
+        "record_id": 42,
+        "user_id": "hobu",
+        "data": "dGhpcyBpcyBzb21lIHRleHQ="
+      })"
+    );
+    doTest(
+      R"({
+        "description": "A description under 32 bytes",
+        "record_id": 42,
+        "data": "dGhpcyBpcyBzb21lIHRleHQ="
+      })"
+    );
+    doTest(
+      R"({
+        "description": "A description under 32 bytes",
+        "record_id": 42,
+        "user_id": "A userID that's way too long",
+        "data": "dGhpcyBpcyBzb21lIHRleHQ="
+      })"
+    );
+    doTest(
+      R"({
+        "description": "A description under 32 bytes",
+        "record_id": 42,
+        "user_id": "hobu"
+      })"
+    );
+    doTest(
+      R"({
+        "description": "A description under 32 bytes",
+        "record_id": 42.234,
+        "user_id": "hobu",
+        "data": "dGhpcyBpcyBzb21lIHRleHQ="
+      })"
+    );
+    doTest(
+      R"({
+        "description": "A description under 32 bytes",
+        "record_id": 422340,
+        "user_id": "hobu",
+        "data": "dGhpcyBpcyBzb21lIHRleHQ="
+      })"
+    );
+    doTest(
+      R"({
+        "description": "A description under 32 bytes",
+        "record_id": 42,
+        "user_id": "hobu",
+        "data": "dGhpcyBpcyBzb21lIHRleHQ="
+      } junk after valid VLR )"
+    );
+}
+
+TEST(LasWriterTest, evlroffset)
+{
+    std::string outfile(Support::temppath("evlr.las"));
+
+    FileUtils::deleteFile(outfile);
+    {
+        LasWriter w;
+        Options wo;
+        std::vector<uint8_t> largeVlr(66000);
+        std::string vlr =
+            " { \"description\": \"A description under 32 bytes\", "
+            "\"record_id\": 42, \"user_id\": \"hobu\", \"data\": \"" +
+            Utils::base64_encode(largeVlr) + "\" }";
+        wo.add("vlrs", vlr);
+        wo.add("minor_version", 4);
+        wo.add("filename", outfile);
+        w.setOptions(wo);
+
+        PointTable t;
+        w.prepare(t);
+        w.execute(t);
+        LasTester tester;
+        LasHeader* h = tester.header(w);
+        // No points in the file
+        EXPECT_EQ(h->eVlrOffset(), h->pointOffset());
+        EXPECT_EQ(h->eVlrCount(), 1u);
+    }
+}
 
 // Make sure that we can forward the LAS_Spec/3 VLR
 TEST(LasWriterTest, forward_spec_3)
@@ -1313,7 +1466,7 @@ TEST(LasWriterTest, issue1940)
 }
 
 // Make sure that we can forward scale from multiple files if they match.
-TEST(LasWRiterTest, issue2663)
+TEST(LasWriterTest, issue2663)
 {
     std::string outfile(Support::temppath("out.las"));
     {
@@ -1354,7 +1507,6 @@ TEST(LasWRiterTest, issue2663)
     EXPECT_EQ(h.scaleZ(), .001);
 }
 
-
 #if defined(PDAL_HAVE_LASZIP)
 // Make sure that we can translate this special test data to 1.4, dataformat 6.
 TEST(LasWriterTest, issue2320)
@@ -1398,3 +1550,50 @@ TEST(LasWriterTest, issue2320)
 }
 #endif
 
+TEST(LasWriterTest, synthetic_points)
+{
+    using namespace Dimension;
+
+    const std::string FILENAME(Support::temppath("synthetic_test.las"));
+    PointTable table;
+
+    table.layout()->registerDims({Id::X, Id::Y, Id::Z, Id::Classification});
+
+    BufferReader bufferReader;
+
+    PointViewPtr view(new PointView(table));
+    view->setField(Id::X, 0, 1.0);
+    view->setField(Id::Y, 0, 2.0);
+    view->setField(Id::Z, 0, 3.0);
+    view->setField(Id::Classification, 0, ClassLabel::Ground | ClassLabel::Synthetic);
+    bufferReader.addView(view);
+
+    Options writerOps;
+    writerOps.add("filename", FILENAME);
+
+    LasWriter writer;
+    writer.setOptions(writerOps);
+    writer.setInput(bufferReader);
+
+    writer.prepare(table);
+    writer.execute(table);
+
+    Options readerOps;
+    readerOps.add("filename", FILENAME);
+
+    PointTable readTable;
+
+    LasReader reader;
+    reader.setOptions(readerOps);
+
+    reader.prepare(readTable);
+    PointViewSet viewSet = reader.execute(readTable);
+    EXPECT_EQ(viewSet.size(), 1u);
+    view = *viewSet.begin();
+    EXPECT_EQ(view->size(), 1u);
+    EXPECT_EQ(ClassLabel::Ground | ClassLabel::Synthetic, view->getFieldAs<uint8_t>(Id::Classification, 0));
+
+    FileUtils::deleteFile(FILENAME);
+}
+
+
